/*

Program:  TLCD.c
Purpose:  Usage of HD44780 2x16 TLCD with PIC18F4550.
Compiler: MPLAB C18 v.3.30 Student Edition in MPLAB IDE v.8.20.

Author:   Predrag Pei (eng. Peshich), Belgrade, Serbia.
Board:    EasyPIC3 by Mikroelektronika with PIC18F4550 on 8MHz.
Date:     March 2009.

This program is made for educational purposes. Free to use and modify.

First part of program: Invoking pause (delay) in program for debugging.

How the function (suppose) is working: Eanbling the Timer2 and assuming that Mikroelektronika`s EasyPIC3 is clocked by 8MHz primary crystall osc. the source for Timer2 is Fosc/4 which means that the incoming clock stream is divided by 4. Because we are using HSPLL the controller and periferals are clocked by 48Mhz, the timer TMR2 is clocked by 12MHz clock stream. This stream enters in prescaler which divides it by 16. This produces 750kHz clock stream which enters in comparator.
The PR2 register is set to 0xFA (250 dec.) which means that comparator will produce one pulse on every 250 pulses from timer TMR2. The result is that that we have 3kHz outcoming stream of pulses from comparator to postscaler which is configured to 1:3 to produce the stream to TMR2IF (interrupt flag).
By this way we have 1mS pulse on TMR2IF!
On every occurence of interrupt from Timer2 we increment Sec for ammount of 0.001 to count the seconds.

*/

#include "p18f4550.h"
#include "delays.h"
#include "Config_Bits.h"
#include "TLCD_Library.h"
#include "Pause.h"

        
volatile float Sec=0.0;		                // Global variable. It`s value can be changed from anywhere in the program.

#pragma code Address_Of_HISR_Is_At=0x08		// We shall use high-interrupt. The address of high-interrupt vector (HISR) is at address 0x08.
void On_Timer2_Interrupt (void)						// On every occurence of interrupt from Timer2...
{
	_asm
	goto	Timer2													  // ... jump to high-interrupt ISR Timer2!
	_endasm
}
#pragma code                              // Return from #pragma to code section.


#pragma interrupt Timer2						// Tell the compiler that ISR for high-interrupt is Timer2.
void Timer2(void)										// The body of high-interrupt ISR starts here. 
{
	PIR1bits.TMR2IF=0;								// Immediately clear the interrupt flag.			
		Sec+=0.001;                     // Increase Sec in miliseconds. Remember, this means that one mS is passed!
}


void Pause(float Seconds)
{
  Sec=0.0;                  // Initialise necessery counters to zero.
	IPR1bits.TMR2IP=1;				// Interrupt priority bit set to 1 - high priority.
	PIE1bits.TMR2IE=1;				// Interrupt on TMR2IF enabled.
	 
	T2CON=0b00010110;					// Timer2 config register:	prescaler (bits 0-1):				divide by 16
	 													//													timer2 ON/OFF (bit 2):			TMR2ON=1 - ON
	 													// 													postscaler (bits 3-7):			1:3
	PR2=0xFA;									// Period register set to 250 (dec.).
	INTCONbits.PEIE=1;				// Enable peripheral interrupt sources.
	INTCONbits.GIE=1;					// Enable global interrupts.
	while(Sec<=Seconds);		  // Wait for the given seconds...
	T2CONbits.TMR2ON=0;				// ... than stop theTimer2, TMR2ON=0 - OFF...
		INTCONbits.PEIE=0;			// ... disable peripheral interrupt sources...
		INTCONbits.GIE=0;				// ... and disble global interrupts.	
}


// Second part of program: Doing something on TLCD using functions defined in file TLCD_Library.c.

// Some text for TLCD.
const unsigned rom char *T1="Hello! I am TLCD",
                        *T2="Line 2, pl.2",         // Line two, third place from the beginig of line.
                        *Cursor_On="Cursor is ON ",
                        *Prompt="Pause 5 sec.",
                        *T3="Line 1, place 0",
                        *T4="Line 2, pl.3",
                        *Cursor_Blink_On1="Cursor is",
                        *Cursor_Blink_On2="blinking... ",
                        *Cursor_Go_Home="... and go HOME!",
                        *Cursor_Home="Cusor go HOME!",
                        *TLCD_Off_1="I will be OFF",
                        *TLCD_Off_2="next 10 sec.!",
                        *TLCD_On_1="Hi!",
                        *TLCD_On_2="I am ON again!",
                        *Cursor_Blink_Off1="Cursor is not",
                        *Cursor_Blink_Off2="blinking! ",
                        *The_End="THE END",
                        *Smiley=":-)";


void main()
{
 unsigned char k=0, prolaz=0x30;
 
 TRISB=0x00; LATB=0x00; TRISD=0x00; LATD=0x00;  // Clear all dependent ports.
 
 	Init_TLCD();                    // Initialise TLCD...
  Cursor_OFF();                   // ... turn cursor OFF...
  Clear_TLCD();                   // ... clear it....
    Text_TLCD(1,0,T1);            // ... and write string T1 on the first line at place 1 from the begining...
    Text_TLCD(2,2,T2);            // ... than write string T2 on the second line at place 3 from the begining...
  Pause(4.0);                     // ... and wait 4 seconds...
  Clear_TLCD();                   // ... etc. ...
    Text_TLCD(1,0,Cursor_On);
    Cursor_ON();
  Pause(4.0);
  Clear_TLCD();
    Text_TLCD(1,2,Prompt);
  Pause(5.0);
	Clear_TLCD();
		Text_TLCD(1,0,T3);
		Text_TLCD(2,2,T4);
	Pause(4.0);
	Clear_TLCD();
	Cursor_Blink_ON();
	  Text_TLCD(1,1,Cursor_Blink_On1);
	  Text_TLCD(2,1,Cursor_Blink_On2);
	Pause(4.0);
	Clear_TLCD();
	  Text_TLCD(2,0,Cursor_Home);
	Pause(4.0);
	Cursor_HOME();
	Pause(4.0);
	Clear_TLCD();
		Text_TLCD(1,2,TLCD_Off_1);
		Text_TLCD(2,0,TLCD_Off_2);
	Pause(4.0);
	TLCD_OFF();
	Pause(10.0);
	TLCD_ON();
	Clear_TLCD();
	  Text_TLCD(1,5,TLCD_On_1);
    Text_TLCD(2,0,TLCD_On_2);
  Pause(4.0);
  Clear_TLCD();
	Cursor_Blink_OFF();
	  Text_TLCD(1,1,Cursor_Blink_Off1);
	  Text_TLCD(2,1,Cursor_Blink_Off2);
	Pause(4.0);
	Clear_TLCD();
	Cursor_OFF();
	  Text_TLCD(1,5,The_End);
	  Text_TLCD(2,7,Smiley);	

while(1);                            // Do nothing.
}
