#include "delays.h"
#include "TLCD_Library.h"

/*==========================================================================================

   On EasyPIC3 developing system when the TLCD is plugged in the upper left connector on board, it works in so-called 4-bit mode.
   This means that if we want to send DATA or INSTRUCTION to TLCD we are using only 4 pins connected to port B:
      - D7 at RB7,
      - D6 at RB6,
      - D5 at RB5 and
      - D4 at RB4.
  This means, further, that instructions or data (bytes) can be sent only by 4 bits (half of a byte) called nibbles. For this reason bytes must be sent in nibbles with the UPPER NIBBLE FIRST (four upper bits of byte - 7,6,5 and 4) and then LOWER NIBBLE (four lower bits of byte - 3,2,1 and 0).
  ATTENTION!
  Pin R/W is used to define the direction of data depending if we want to read from or to write to TLCD. On EasyPIC3 this pin is connected   to ground (logical 0) so WE CAN ONLY WRITE DATA TO TLCD! Because of that this pin is not used by microcontroller in this program.
  
  One another explanation:
  
  The initialisation of TLCD needs that data 0x30 must be written to TLCD THREE times one by one. BYTE will be sent by pins of port B and
  TLCD will take only bits that are available on pins RB7, RB6, RB5 and RB4 (upper nibble - 0x3). Data available on pins RB3, RB2, RB1 and
  RB0 (lower nibble - 0x0) WILL BE IGNORED so we do not write them to port B!

  Definition of pins:
  D7 - D0	- lines for transferring data.
  E 		  - ENABLE. Logical "1" prepares TLCD for writting operation. Transition from high to low (or from "1" to "0") actually writes data
            into the  TLCD.
  RW 		  - READ/WRITE.  Logical "1" prepares TLCD to give us data (READ signal). Logical "0" prepares TLCD to receive data from us (WRITE            signal). Remember, in this program this signal is always 0 - write operation!
  RS 		  - toggle between registers DATA/INSTRUCTION. Logical "1" means that next byte (or two nibbles) is/are DATA. Logical "0" means that 
            that next byte (or two nibbles) is/are INSTRUCTION.
            be written.

*/
void Init_TLCD(void)
{
     WRITE_INSTRUCTION;         // Tell TLCD that the instruction will be written.
     Delay10KTCYx(255);         // Wait 45ms for power to be stabilised...
          PORTBbits.RB7=0;      // ... and than send 0x30 for the first time to port B.
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=1;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;          // ... and than put ENABLE to logical 0 to write the data.

     Delay1KTCYx(20);           // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // ... and than send 0x30 for the second time to port B.
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=1;
              TO_BE_WRITTEN;   // Than rise ENABLE bit...
              WRITTEN;         // ... and than put ENABLE to logical 0 to write the data.
     Delay1KTCYx(20);          // Than, again, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;     // ... and than send 0x30 for the third time to port B.
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=1;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

/*
First step of initialisation is finished. Now, send nibble 0x2 to TLCD to tell it that it will work in 4-bit mode.
*/
     Delay1KTCYx(20);           // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // Than, send 0x2 to TLCD.
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=0;;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

/*
We are working with 2-line display. Send 0x28 to TLCD:
*/

      Delay1KTCYx(20);          // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // First, upper nibble - 0x2...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

      Delay1KTCYx(20);          // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=1;      // ... than lower nibble - 0x8.
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

/*
To turn OFF TLCD send 0x08:
*/

      Delay1KTCYx(20);          // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // First, upper nibble - 0x0...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

      Delay1KTCYx(20);          // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=1;      // ... than lower nibble - 0x8...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

 /*
 To clear TLCD send 0x01:
 */
    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // First, upper nibble - 0x0...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // ... than lower nibble - 0x1...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=1;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

/*
For Entry Mode Set send 0x06:
*/
    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // First, upper nibble - 0x0...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
	        PORTBbits.RB7=0;      // ... than lower nibble - 0x6...
	        PORTBbits.RB6=1;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.



/*
To turn TLCD ON send 0x0F:
*/

    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=0;      // First, upper nibble - 0x0...
	        PORTBbits.RB6=0;
	        PORTBbits.RB5=0;
	        PORTBbits.RB4=0;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.

    Delay1KTCYx(20);            // Than, wait 1.2 msec for the previous instruction to finish in TLCD...
          PORTBbits.RB7=1;      // ... than lower nibble - 0xF...
	        PORTBbits.RB6=1;
	        PORTBbits.RB5=1;
	        PORTBbits.RB4=1;
              TO_BE_WRITTEN;    // Than rise ENABLE bit...
              WRITTEN;       // ... and than put ENABLE to logical 0 to write the data.
}

/******************************************************************************
 *Function:             void Text_TLCD (unsigned char Line, unsigned char Position, const unsigned rom char *Text)
 *
 *
 *Entry:                Line       - line of text to be written.
                        Position  - starting place from the left side (begining) of line.
                        *Text     - pointer to text string.
 *
 *
 *Explanation:			    This function writes text offseted from the begining of line in the specified row. 
                        One explanation:
                        To write text on TLCD we must take two steps:
                          *First step:
                            - write in TLCD one byte composed of bit D7 (1-INSTRUCTION) plus 7-bit address (D6-D0)
                              for the place of character.
                          *Second step:
                            - than, write to TLCD the 8-bit code of character (DATA).
                        Addresses of the first places on the lines are:
                            - 0x00 for the first place (character) on first line
                            - 0x40 for the first place (character) on second line.
                        
                        Let`s go on first step in detail.
                        
                        Here is the "egsotic" of the first step: to write the character at some place offseted
                        from the begining of line, the combination of bytes will be written: bit D7 0b1000 0000 (0x80) which
                        is the instruction to TLCD that address will be written, plus byte of address of the first character on first
                        line (0x00), plus byte of the postion of character offseted from the begining of first line of TLCD.
                        
                                                0x80 + 0x00 + position.
                                                
                        For the second row the procedure is the same as the previous one: bit D7 0b1000 0000 (0x80) which
                        is the instruction to TLCD that address will be written, plus byte of address of the first character on second
                        line (0x40), plus byte of the postion of character offseted from the begining of second line of TLCD.
                        
                        												0x80 + 0x40 + position = 0xC0 + position.
                        
                        Now, the second step in detail.
                        
                            - Calculate the postion of character from the beginig on line.                        
                            - Take the character code from text string trough the while()-loop. Character code is saved to variable Char.
                              Than, variable Char is sent to port B (RB7-RB4). Because the TLCD is in 4-bit mode only the upper nibble
                              will be written to TLCD.
                            - Than, take the variable Char again and shift it left 4 places because we need the lower nibble of the
                              variable Char and send it to TLCD via port B.
                        Now, the TLCD by itself will write the character offseted from the begining at the desired line and increments
                        cursor automatically to next place. The procedure is repeated untill the end of string (0x00).
 */
  
 /*****************************************************************************/

void Text_TLCD (unsigned char Line, unsigned char Position, const unsigned rom char *Text)
{
 unsigned char i=0, address_1=0, address_2=0, Char;
 

// Calculating the starting position (offset) of first character on line.

  address_1=0x80 + Position;    // We want to write on the first line plus the offset from begining and counts places from position 0.
  address_2=0xC0 + Position;    // We want to write on the second line plus the offset from begining and counts places from position 0.

  switch(Line)                  // First, determine in which line we want to write.
  {
  case 1: PORTB=address_1;						// Write in the first part of address in port B. Only the upper nibble will be written to TLCD.	
              TO_BE_WRITTEN;     			// Than rise ENABLE bit...
                Delay1KTCYx(25);    	// ... wait some time for the signal to stabilise (experiment with this; maybe it could be shorter)...
              WRITTEN;          		  // ... than write the upper nibble.
          address_1=address_1<<4;			// Shift the address byte for 4 places to the left to exchange the upper nibble with lower nibble.
          PORTB=address_1;            // Again, write in the second part of address in port B (lower nibble).
              TO_BE_WRITTEN;          // Than rise ENABLE bit...
                Delay1KTCYx(25);      // ... wait some time for the signal to stabilise (experiment with this; maybe it could be shorter)...
              WRITTEN;                // ... than write the upper nibble to TLCD.
          break;
  case 2: PORTB=address_2;            // Write in the first part of address in port B. Only the upper nibble will be written to TLCD.
              TO_BE_WRITTEN;          // Than rise ENABLE bit...
                Delay1KTCYx(25);      // ... wait some time for the signal to stabilise...
              WRITTEN;                // ... than write the upper nibble.
          address_2=address_2<<4;			// Shift the address byte for 4 places to the left to exchange the upper nibble with lower nibble.
          PORTB=address_2;            // Again, write in the second part of address in port B (lower nibble).
              TO_BE_WRITTEN;          // Than rise ENABLE bit...
               Delay1KTCYx(25);       // ... wait some time for the signal to stabilise...
              WRITTEN;                // ... than write the upper nibble to TLCD.
          break;
  }
  
  // Taking characters from string.

  while(Text[i]!=0x00)  			// Check if we reach the end of string: terminator 0x00. And if we are not...
 {
  Char=Text[i];       			  // ... take the character from position [i]...
  PORTB=Char;          			  // ... and write it to port B.
    PORTBbits.RB2=1;       		// Now, tell the TLCD that we are going to write data to it. Only the upper nibble will be written.
    TO_BE_WRITTEN;       			// Than rise ENABLE bit...
          Delay1KTCYx(25); 		// ... wait some time for the signal to stabilise (experiment with this; maybe it could be shorter)...
          WRITTEN;       	    // ... than write the upper nibble.
  Char=Text[i]<<4;    			  // Shift the byte Char for 4 places to the left to exchange the upper nibble with lower nibble...
  PORTB=Char;          			  // ... and write it to port B.
  PORTBbits.RB2=1;       	  	// Again, tell the TLCD that we are going to write data to it because the shift operation clears that bit.
                              // Again, only the upper nibble will be written (and that is, actually, the lower nibble).
    TO_BE_WRITTEN;       			// Than rise ENABLE bit...
          Delay1KTCYx(25);  	// ... wait some time for the signal to stabilise (experiment with this; maybe it could be shorter)...
          WRITTEN;        	  // ... than write the upper nibble.
      i++;              			// Increment "i" to show to the next character in string (for the next character position on TLCD).
  }
  PORTBbits.RB2=0;          	// Down the RB2 signal for any case.
}


//In the following code there are the functions for some operations on TLCD.


/******************************************************************************
 *Function:        		  void Interface_4bit(void)
 *
 *Entry:           		  0x2
 *
 *Explanation:			    This function prepares PORTBbits for the upper nibble for 4-bit mode operation.
 *
 *****************************************************************************/
void Interface_4bit(void)
{
	PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=1;
	PORTBbits.RB4=0;
}
// The end of function void Interface_4bit(void)


/******************************************************************************
 *Function:        		  void Interface_8bit(void)
 *
 *Entry:           		  0x3
 *
 *Explanation:			    This function prepares PORTBbits for the upper nibble for 8-bit mode operation.

 *****************************************************************************/

void Interface_8bit(void)
{
	PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=1;
	PORTBbits.RB4=1;
}
// The end of function void Interface_8bit(void)


/******************************************************************************
 *Function:        		  void One_Line_TLCD(void)
 *
 *Entry:           		  0x20
 *
 *Explanation:			    This function prepares byte on PORTBbits for type of TLCD: one-line display.

 *****************************************************************************/
void One_Line_TLCD(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
	PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=1;
	PORTBbits.RB4=0;
     TO_BE_WRITTEN;   // Rise ENABLE bit...
     WRITTEN;         // ... than write the upper nibble.

 Delay1KTCYx(20);
	PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the lower nibble.
}
// The end of function void One_Line_TLCD(void)



/******************************************************************************
 *Function:        		  void Two_Line_TLCD(void)
 *
 *Entry:           		  0x28
 *
 *Explanation:			    This function prepares byte on PORTBbits for type of TLCD: two-line display.

 *Primedba:            	Nema
 *****************************************************************************/
void Two_Line_TLCD(void)
{
 WRITE_INSTRUCTION;
Delay1KTCYx(20);
	PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=1;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

Delay1KTCYx(20);
	PORTBbits.RB7=1;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the lower nibble.
}
// The end of function void Dvolinijski_Displ(void)



/******************************************************************************
 *Function:        		  void TLCD_ON(void)
 *
 *Entry:           		  0x0C
 *
 *Explanation:			    This function prepares byte on PORTBbits to swtch TLCD ON.

 *****************************************************************************/
void TLCD_ON(void)
{
 WRITE_INSTRUCTION;
Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
	PORTBbits.RB7=1;
	PORTBbits.RB6=1;     // 1 - display ON, 0 - display OFF
	PORTBbits.RB5=1;     // 1 - cursor ON, 0 - cursor OFF
	PORTBbits.RB4=0;     // 1 - blink ON, 0 - blink OFF
	    TO_BE_WRITTEN;   // Rise ENABLE bit...
      WRITTEN;      // ... than write the lower nibble.
}
// The end of function void TLCD_ON(void)


/******************************************************************************
 *Function:        		  void TLCD_OFF(void)
 *
 *Entry:           		  0x08
 *
 *Explanation:			    This function prepares byte on PORTBbits to swtch TLCD OFF.

 *****************************************************************************/
void TLCD_OFF(void)
{
 WRITE_INSTRUCTION;
Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
	PORTBbits.RB7=1;
	PORTBbits.RB6=0;    // 1 - display ON, 0 - display OFF
	PORTBbits.RB5=0;    // 1 - cursor ON, 0 - cursor OFF
	PORTBbits.RB4=0;    // 1 - blink ON, 0 - blink OFF
	    TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the lower nibble.
}
// The end of function void TLCD_OFF(void)



/******************************************************************************
 *Function:        		  void Clear_TLCD()(void)
 *
 *Entry:           		  0x01
 *
 *Explanation:			    This function prepares byte on PORTBbits to clear TLCD.

 *****************************************************************************/
void Clear_TLCD(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=1;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the lower nibble.
}
// The end of function void Clear_TLCD()_MSB(void)



/******************************************************************************
 *Function:        		  void Cursor_ON(void)
 *
 *Entry:           		  0x02
 *
 *Explanation:			    This function prepares byte on PORTBbits to swtch cursor ON.

 *****************************************************************************/
void Cursor_ON(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
  PORTBbits.RB7=1;
	PORTBbits.RB6=1;
	PORTBbits.RB5=1;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;   // Rise ENABLE bit...
      WRITTEN;      // ... than write the lower nibble.
}
// The end of function void Cursor_ON(void)



/******************************************************************************
 *Function:        		  void Cursor_OFF(void)
 *
 *Entry:           		  0x00
 *
 *Explanation:			   This function prepares byte on PORTBbits to swtch cursor OFF.

 *****************************************************************************/
void Cursor_OFF(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
  PORTBbits.RB7=1;
	PORTBbits.RB6=1;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.
}
// The end of function void Cursor_OFF(void)


/******************************************************************************
 *Function:        		  void Cursor_Blink_ON(void)
 *
 *Entry:           		  0x00
 *
 *Explanation:			   This function prepares byte on PORTBbits for cursor to blink.

 *****************************************************************************/
void Cursor_Blink_ON(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
  PORTBbits.RB7=1;
	PORTBbits.RB6=1;
	PORTBbits.RB5=1;
	PORTBbits.RB4=1;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.
}
// The end of function void Cursor_Blink_ON(void)


/******************************************************************************
 *Function:        		  void Cursor_Blink_OFF(void)
 *
 *Entry:           		  0x00
 *
 *Explanation:			   This function prepares byte on PORTBbits for cursor not to blink.

 *****************************************************************************/
void Cursor_Blink_OFF(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
  PORTBbits.RB7=0;
	PORTBbits.RB6=0;
	PORTBbits.RB5=0;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
  PORTBbits.RB7=1;
	PORTBbits.RB6=1;
	PORTBbits.RB5=1;
	PORTBbits.RB4=0;
      TO_BE_WRITTEN;  // Rise ENABLE bit...
      WRITTEN;        // ... than write the upper nibble.
}
// The end of function void Cursor_Blink_ON(void)



/******************************************************************************
 *Function:        		  void Cursor_To_Begining(void)
 *
 *Entry:           		  0x02
 *
 *Explanation:			    This function prepares byte on PORTBbits to move cursor to the beginig of line.

 *****************************************************************************/

void Cursor_HOME(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
	  PORTBbits.RB7=0;
	  PORTBbits.RB6=0;
	  PORTBbits.RB5=0;
	  PORTBbits.RB4=0;
        TO_BE_WRITTEN;  // Rise ENABLE bit...
        WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
	  PORTBbits.RB7=0;
	  PORTBbits.RB6=0;
	  PORTBbits.RB5=1;
	  PORTBbits.RB4=0;
        TO_BE_WRITTEN;  // Rise ENABLE bit...
        WRITTEN;        // ... than write the upper nibble.
}
// The end of function void Cursor_To_Begining(void).


/******************************************************************************
 *Function:        		  void Entry_Mode_LSB(void)
 *
 *Entry:           		  0x06
 *
 *Explanation:			    This function prepares byte on PORTBbits for Entry mode.
 
 *****************************************************************************/
void Entry_Mode(void)
{
 WRITE_INSTRUCTION;
 Delay1KTCYx(20);
	  PORTBbits.RB7=0;
	  PORTBbits.RB6=0;
	  PORTBbits.RB5=0;
	  PORTBbits.RB4=0;
        TO_BE_WRITTEN;  // Rise ENABLE bit...
        WRITTEN;        // ... than write the upper nibble.

 Delay1KTCYx(20);
	  PORTBbits.RB7=0;
	  PORTBbits.RB6=1;
	  PORTBbits.RB5=1;
	  PORTBbits.RB4=0;
        TO_BE_WRITTEN;  // Rise ENABLE bit...
        WRITTEN;        // ... than write the upper nibble.
}
// The end of function void Entry_Mode(void)
